from flask import flash, redirect, render_template, request, url_for
from flask_breadcrumbs import register_breadcrumb
from app import db, hashids
from app.models import Corpus, Role, User, UserSettingJobStatusMailNotificationLevel
from app.settings.forms import EditNotificationsForm
from app.settings.forms import EditAccountForm
from . import bp
from .forms import AdminEditUserForm
from app.users.utils import (
    user_endpoint_arguments_constructor as user_eac,
    user_dynamic_list_constructor as user_dlc
)


@bp.route('')
@register_breadcrumb(bp, '.', '<i class="material-icons left">admin_panel_settings</i>Administration')
def admin():
    return render_template(
        'admin/admin.html.j2',
        title='Administration'
    )


@bp.route('/corpora')
@register_breadcrumb(bp, '.corpora', 'Corpora')
def corpora():
    corpora = Corpus.query.all()
    return render_template(
        'admin/corpora.html.j2',
        title='Corpora',
        corpora=corpora
    )


@bp.route('/users')
@register_breadcrumb(bp, '.users', '<i class="material-icons left">group</i>Users')
def users():
    users = User.query.all()
    return render_template(
        'admin/users.html.j2',
        title='Users',
        users=users
    )


@bp.route('/users/<hashid:user_id>')
@register_breadcrumb(bp, '.users.entity', '', dynamic_list_constructor=user_dlc)
def user(user_id):
    user = User.query.get_or_404(user_id)
    return render_template(
        'admin/user.html.j2',
        title=user.username,
        user=user
    )


@bp.route('/users/<hashid:user_id>/edit', methods=['GET', 'POST'])
@register_breadcrumb(bp, '.users.entity.edit', 'Edit', endpoint_arguments_constructor=user_eac)
def edit_user(user_id):
    user = User.query.get_or_404(user_id)
    admin_edit_user_form = AdminEditUserForm(
        data={'confirmed': user.confirmed, 'role': user.role.hashid},
        prefix='admin-edit-user-form'
    )
    edit_profile_settings_form = EditAccountForm(
        user,
        data=user.to_json_serializeable(),
        prefix='edit-profile-settings-form'
    )
    edit_notification_settings_form = EditNotificationsForm(
        data=user.to_json_serializeable(),
        prefix='edit-notification-settings-form'
    )
    if (admin_edit_user_form.submit.data
            and admin_edit_user_form.validate()):
        user.confirmed = admin_edit_user_form.confirmed.data
        role_id = hashids.decode(admin_edit_user_form.role.data)
        user.role = Role.query.get(role_id)
        db.session.commit()
        flash('Your changes have been saved')
        return redirect(url_for('.edit_user', user_id=user.id))
    if (edit_profile_settings_form.submit.data
            and edit_profile_settings_form.validate()):
        user.email = edit_profile_settings_form.email.data
        user.username = edit_profile_settings_form.username.data
        db.session.commit()
        flash('Your changes have been saved')
        return redirect(url_for('.edit_user', user_id=user.id))
    if (edit_notification_settings_form.submit.data
            and edit_notification_settings_form.validate()):
        user.setting_job_status_mail_notification_level = \
            UserSettingJobStatusMailNotificationLevel[
                edit_notification_settings_form.job_status_mail_notification_level.data  # noqa
            ]
        db.session.commit()
        flash('Your changes have been saved')
        return redirect(url_for('.edit_user', user_id=user.id))
    return render_template(
        'admin/edit_user.html.j2',
        admin_edit_user_form=admin_edit_user_form,
        edit_profile_settings_form=edit_profile_settings_form,
        edit_notification_settings_form=edit_notification_settings_form,
        title='Edit user',
        user=user
    )
